library DateiSpeicher_o0;

// Delphi 5 DLL-source (DateiSpeicher_o0.DLL) fr PofiLab   kpaul 02.09.2007
// die Werte der 16 Eingnge werden bei fallender Flanke an WR in eine
// Datei (.ini , .txt o..) geschrieben
// bei fallender Flanke an RD werden die 16 Werte aus der Datei gelesen
// und auf die Ausgnge gelegt

// es kann keine 0 in die Datei geschrieben werden, wenn dies versucht wird
// geht der entsprechende Ausgang auf 5, der Wert in der Datei bleibt unverndert.

// der gesamte Pfad und Dateiname wird im Eigenschaften-Dialog der DLL unter
// [Einstellungen...] angegeben z.B.  D:\Programme\ProfiLab-Expert40\Data\AnalogSp1.ini
// es knnen mehrere DLLs in einem Projekt betrieben werden, natrlich mssen unterschiedliche
// Dateinamen eingegeben werden!
// Funktioniert auch in compilierten Projekten.


uses
  SysUtils,
  Dialogs,
  Windows,
  Classes,
  StrUtils,
  IniFiles;

{$R *.RES}

Const  Inputs  = 18;  // number of inputs
       Outputs = 16;  // number of outputs

       {INPUTS}
     //W0..15       // index of input variable W0..15
       WR = 16;     // index of input variable Write
       RD = 17;     // index of input variable Read

       {OUTPUTS}
     //R0..15       // index of output variable R0..15

       {USER}
       StringLaenge = 0;    // index user variable StringLaenge
    // (PfadDateiStr  1-70) // Array fr Ascii-Werte
       WR_old = 72;         // index user variable WR_old
       RD_old = 73;         // index user variable RD_old
       DLLIndex = 100;      // DLL No. set by ProfiLab


Type TDLLParams = array[0..100] of extended;   //Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

function NumInputs: Byte;
begin
  result:=Inputs; //Define number of component input pins
end;

function NumOutputs: Byte;
begin
  result:=Outputs; //Define number of component output pins
end;

Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
   result:='W'+intToStr(Channel);   //"W0".."W15" (Bezeichnung der Eingnge)
   case Channel of
   WR:   result:='|WR';  //  "|WR"  (IN)
   RD:   result:='|RD';  //  "|RD"  (IN)
   end;
end;

Function OutputName(Channel: Byte): ShortString; // Return name for each component output pin
begin
   result:='R'+intToStr(Channel);   //"R0".."R15" (Bezeichnung der Ausgnge)
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams); //called when ProfiLab enters RUN mode
begin
  // nothing to be done
end;

Procedure SimStop(PInput,POutput,PUser: PDLLParams); //called when ProfiLab RUN mode is terminated
begin
  // nothing to be done
end;

Procedure Calculate(PInput,POutput,PUser: PDLLParams); //called regularly from ProfiLab
var i: Integer;
    W, R, W_old, R_old: Boolean;
    IniFile: TIniFile;

  Function PfadDateiStr: string;  //String aus Array-Ascii-Werte zusammenstellen
  var i: integer;
      s: String;
  begin
    s:= '';                                     //erst lschen
    for i:=1 to round(PUser^[StringLaenge]) do
      s:=s + CHR(round(PUser^[i]));
    result:=s;                                  //Ergebnisbergabe
  end;

begin

  W:= PInput^[WR] >2.5;
  R:= PInput^[RD] >2.5;
  W_old:= PUser^[WR_old] >2.5;
  R_old:= PUser^[RD_old] >2.5;

  // Werte der Input-Pins in Datei schreiben:
  if (not W) and W_old then                     //check out falling edge at WR
  begin
     IniFile:=TIniFile.Create(PfadDateiStr);
     For i:=0 to 15 do
     begin
       if PInput^[i] <> 0 then          //0 wird nicht in Datei geschrieben            0 voran bei 1-stellig
         IniFile.WriteFloat('DateiSpeicher-DLL'+IntToStr(round(PUser^[100])),RightStr('0'+intToStr(i),2),PInput^[i])
       else POutput^[i]:= 5;            //stattdessen geht Ausgang auf High
     end;
     IniFile.Free;
  end;
  // Werte fr Output-Pins aus Datei lesen:
  if (not R) and R_old then                     //check out falling edge at RD
  begin
     IniFile:=TIniFile.Create(PfadDateiStr);                                                //DLL-Nr.            0 voran bei 1-stellig
     For i:=0 to 15 do POutput^[i]:=IniFile.ReadFloat('DateiSpeicher-DLL'+IntToStr(round(PUser^[100])),RightStr('0'+intToStr(i),2),0);
     IniFile.Free;
  end;

  PUser^[WR_old]:= PInput^[WR];                 //merken
  PUser^[RD_old]:= PInput^[RD];                 //merken

end;        //Procedure Calculate

Procedure Configure(PUser: PDLLParams);
var i: integer;
    PfadDateiStr: String;

begin
  //---------- String aus Array-Ascii-Werte zusammenstellen:
  PfadDateiStr:= '';            //erst lschen
  for i:=1 to round(PUser^[StringLaenge]) do
    PfadDateiStr:=PfadDateiStr + CHR(round(PUser^[i]));
  //------------------------------------------------------------

  PfadDateiStr:= InputBox('Input Box', 'Pfad\Dateiname eingeben:', PfadDateiStr); //Input mit Vorgabe des alten Strings
  if (length(PfadDateiStr) > 70) or (length(PfadDateiStr) < 1) then     //max. Lnge des PfadDateiStrings 70 Zeichen
  begin
    ShowMessage ('Pfad\Dateiname zw. 1-70 Zeichen !');
    exit;
  end;
  PUser^[StringLaenge]:= length(PfadDateiStr);                          //Lnge bergeben

  //---------- String in Ascii-Werte zerlegen und im PUser-Array speichern
  for i:=1 to round(PUser^[StringLaenge]) do
    PUser^[i]:= ORD(PfadDateiStr[i]);
  //------------------------------------------------------------

//  ShowMessage ('String= ' + PfadDateiStr);              //Meldung
end;


//export methods for ProfiLab
exports SimStart,
        SimStop,
        NumInputs,
        NumOutputs,
        Calculate,
        InputName,
        OutputName,
        Configure;

begin
end.

